# -*- coding: utf-8 -*-
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Accesses the google.android.overtheair.v1 OverTheAirService API."""

import pkg_resources
import warnings

from google.oauth2 import service_account
import google.api_core.gapic_v1.client_info
import google.api_core.gapic_v1.config
import google.api_core.gapic_v1.method
import google.api_core.gapic_v1.routing_header
import google.api_core.grpc_helpers
import google.api_core.path_template
import grpc

from google.android.overtheair_v1.gapic import enums
from google.android.overtheair_v1.gapic import over_the_air_service_client_config
from google.android.overtheair_v1.gapic.transports import over_the_air_service_grpc_transport
from google.android.overtheair_v1.proto import overtheair_data_pb2
from google.android.overtheair_v1.proto import overtheair_service_messages_pb2
from google.android.overtheair_v1.proto import overtheair_service_pb2_grpc

_GAPIC_LIBRARY_VERSION = pkg_resources.get_distribution(
    'android-androidovertheair', ).version


class OverTheAirServiceClient(object):
    """
    ## Service for Android Over the Air(OTA).

    ## ----  deployment related rpcs              ---
    """

    SERVICE_ADDRESS = 'androidovertheair.googleapis.com:443'
    """The default address of the service."""

    # The name of the interface for this client. This is the key used to
    # find the method configuration in the client_config dictionary.
    _INTERFACE_NAME = 'google.android.overtheair.v1.OverTheAirService'

    @classmethod
    def from_service_account_file(cls, filename, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
        file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            OverTheAirServiceClient: The constructed client.
        """
        credentials = service_account.Credentials.from_service_account_file(
            filename)
        kwargs['credentials'] = credentials
        return cls(*args, **kwargs)

    from_service_account_json = from_service_account_file

    @classmethod
    def deployment_path(cls, deployment):
        """Return a fully-qualified deployment string."""
        return google.api_core.path_template.expand(
            'deployments/{deployment}',
            deployment=deployment,
        )

    @classmethod
    def config_path(cls, deployment, config):
        """Return a fully-qualified config string."""
        return google.api_core.path_template.expand(
            'deployments/{deployment}/configs/{config}',
            deployment=deployment,
            config=config,
        )

    @classmethod
    def group_path(cls, deployment, group):
        """Return a fully-qualified group string."""
        return google.api_core.path_template.expand(
            'deployments/{deployment}/groups/{group}',
            deployment=deployment,
            group=group,
        )

    @classmethod
    def chart_path(cls, deployment, group, chart):
        """Return a fully-qualified chart string."""
        return google.api_core.path_template.expand(
            'deployments/{deployment}/groups/{group}/charts/{chart}',
            deployment=deployment,
            group=group,
            chart=chart,
        )

    @classmethod
    def member_path(cls, deployment, group, member):
        """Return a fully-qualified member string."""
        return google.api_core.path_template.expand(
            'deployments/{deployment}/groups/{group}/members/{member}',
            deployment=deployment,
            group=group,
            member=member,
        )

    @classmethod
    def package_path_path(cls, deployment, package_path):
        """Return a fully-qualified package_path string."""
        return google.api_core.path_template.expand(
            'deployments/{deployment}/packages/{package_path=**}',
            deployment=deployment,
            package_path=package_path,
        )

    def __init__(self,
                 transport=None,
                 channel=None,
                 credentials=None,
                 client_config=None,
                 client_info=None):
        """Constructor.

        Args:
            transport (Union[~.OverTheAirServiceGrpcTransport,
                    Callable[[~.Credentials, type], ~.OverTheAirServiceGrpcTransport]): A transport
                instance, responsible for actually making the API calls.
                The default transport uses the gRPC protocol.
                This argument may also be a callable which returns a
                transport instance. Callables will be sent the credentials
                as the first argument and the default transport class as
                the second argument.
            channel (grpc.Channel): DEPRECATED. A ``Channel`` instance
                through which to make calls. This argument is mutually exclusive
                with ``credentials``; providing both will raise an exception.
            credentials (google.auth.credentials.Credentials): The
                authorization credentials to attach to requests. These
                credentials identify this application to the service. If none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
                This argument is mutually exclusive with providing a
                transport instance to ``transport``; doing so will raise
                an exception.
            client_config (dict): DEPRECATED. A dictionary of call options for
                each method. If not specified, the default configuration is used.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.
        """
        # Raise deprecation warnings for things we want to go away.
        if client_config is not None:
            warnings.warn(
                'The `client_config` argument is deprecated.',
                PendingDeprecationWarning,
                stacklevel=2)
        else:
            client_config = over_the_air_service_client_config.config

        if channel:
            warnings.warn(
                'The `channel` argument is deprecated; use '
                '`transport` instead.',
                PendingDeprecationWarning,
                stacklevel=2)

        # Instantiate the transport.
        # The transport is responsible for handling serialization and
        # deserialization and actually sending data to the service.
        if transport:
            if callable(transport):
                self.transport = transport(
                    credentials=credentials,
                    default_class=over_the_air_service_grpc_transport.
                    OverTheAirServiceGrpcTransport,
                )
            else:
                if credentials:
                    raise ValueError(
                        'Received both a transport instance and '
                        'credentials; these are mutually exclusive.')
                self.transport = transport
        else:
            self.transport = over_the_air_service_grpc_transport.OverTheAirServiceGrpcTransport(
                address=self.SERVICE_ADDRESS,
                channel=channel,
                credentials=credentials,
            )

        if client_info is None:
            client_info = google.api_core.gapic_v1.client_info.ClientInfo(
                gapic_version=_GAPIC_LIBRARY_VERSION, )
        else:
            client_info.gapic_version = _GAPIC_LIBRARY_VERSION
        self._client_info = client_info

        # Parse out the default settings for retry and timeout for each RPC
        # from the client configuration.
        # (Ordinarily, these are the defaults specified in the `*_config.py`
        # file next to this one.)
        self._method_configs = google.api_core.gapic_v1.config.parse_method_configs(
            client_config['interfaces'][self._INTERFACE_NAME], )

        # Save a dictionary of cached API call functions.
        # These are the actual callables which invoke the proper
        # transport methods, wrapped with `wrap_method` to add retry,
        # timeout, and the like.
        self._inner_api_calls = {}

    # Service calls
    def get_deployment(self,
                       name,
                       view=None,
                       retry=google.api_core.gapic_v1.method.DEFAULT,
                       timeout=google.api_core.gapic_v1.method.DEFAULT,
                       metadata=None):
        """
        Gets a ``Deployment``. Returns ``NOT_FOUND`` if the deployment does not
        exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.deployment_path('[DEPLOYMENT]')
            >>>
            >>> response = client.get_deployment(name)

        Args:
            name (str): Relative resource name. For example: "deployments/deployment\_x"
            view (~google.android.overtheair_v1.types.View): Optional. Specify whether basic or detailed information of the deployment
                should be returned.
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.Deployment` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'get_deployment' not in self._inner_api_calls:
            self._inner_api_calls[
                'get_deployment'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.get_deployment,
                    default_retry=self._method_configs['GetDeployment'].retry,
                    default_timeout=self._method_configs['GetDeployment'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.GetDeploymentRequest(
            name=name,
            view=view,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['get_deployment'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def list_deployments(self,
                         view=None,
                         retry=google.api_core.gapic_v1.method.DEFAULT,
                         timeout=google.api_core.gapic_v1.method.DEFAULT,
                         metadata=None):
        """
        Lists ``deployments``.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> response = client.list_deployments()

        Args:
            view (~google.android.overtheair_v1.types.View): Optional. Specify whether basic or detailed information of each deployment
                should be returned.
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.ListDeploymentsResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'list_deployments' not in self._inner_api_calls:
            self._inner_api_calls[
                'list_deployments'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.list_deployments,
                    default_retry=self._method_configs['ListDeployments'].
                    retry,
                    default_timeout=self._method_configs['ListDeployments'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.ListDeploymentsRequest(
            view=view, )
        return self._inner_api_calls['list_deployments'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def update_deployment(self,
                          name,
                          deployment,
                          version_tag,
                          retry=google.api_core.gapic_v1.method.DEFAULT,
                          timeout=google.api_core.gapic_v1.method.DEFAULT,
                          metadata=None):
        """
        Updates a ``deployment``.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.deployment_path('[DEPLOYMENT]')
            >>>
            >>> # TODO: Initialize `deployment`:
            >>> deployment = {}
            >>>
            >>> # TODO: Initialize `version_tag`:
            >>> version_tag = {}
            >>>
            >>> response = client.update_deployment(name, deployment, version_tag)

        Args:
            name (str): Relative resource name. For example: "deployments/deployment\_x"
            deployment (Union[dict, ~google.android.overtheair_v1.types.Deployment]): Required. Only the deployment description can be updated.

                If a dict is provided, it must be of the same form as the protobuf
                message :class:`~google.android.overtheair_v1.types.Deployment`
            version_tag (Union[dict, ~google.android.overtheair_v1.types.VersionTag]): Required. The deployment to be updated should match this version.
                If another client updates the existing instance concurrently, this
                update operation will fail.

                If a dict is provided, it must be of the same form as the protobuf
                message :class:`~google.android.overtheair_v1.types.VersionTag`
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.Deployment` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'update_deployment' not in self._inner_api_calls:
            self._inner_api_calls[
                'update_deployment'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.update_deployment,
                    default_retry=self._method_configs['UpdateDeployment'].
                    retry,
                    default_timeout=self._method_configs['UpdateDeployment'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.UpdateDeploymentRequest(
            name=name,
            deployment=deployment,
            version_tag=version_tag,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['update_deployment'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def get_package(self,
                    name,
                    view=None,
                    retry=google.api_core.gapic_v1.method.DEFAULT,
                    timeout=google.api_core.gapic_v1.method.DEFAULT,
                    metadata=None):
        """
        Gets a ``package``. Returns ``NOT_FOUND`` if the matching path does not
        exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.package_path_path('[DEPLOYMENT]', '[PACKAGE_PATH]')
            >>>
            >>> response = client.get_package(name)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/packages/package\_a"
            view (~google.android.overtheair_v1.types.View): Optional. Specify whether basic or detailed information of the package
                should be returned.
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.Package` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'get_package' not in self._inner_api_calls:
            self._inner_api_calls[
                'get_package'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.get_package,
                    default_retry=self._method_configs['GetPackage'].retry,
                    default_timeout=self._method_configs['GetPackage'].timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.GetPackageRequest(
            name=name,
            view=view,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['get_package'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def list_packages(self,
                      name,
                      view=None,
                      retry=google.api_core.gapic_v1.method.DEFAULT,
                      timeout=google.api_core.gapic_v1.method.DEFAULT,
                      metadata=None):
        """
        Lists ``packages``.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.deployment_path('[DEPLOYMENT]')
            >>>
            >>> response = client.list_packages(name)

        Args:
            name (str): Relative resource name. For example: "deployments/deployment\_x"
            view (~google.android.overtheair_v1.types.View): Optional. Specify whether basic or detailed information of each package
                should be returned.
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.ListPackagesResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'list_packages' not in self._inner_api_calls:
            self._inner_api_calls[
                'list_packages'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.list_packages,
                    default_retry=self._method_configs['ListPackages'].retry,
                    default_timeout=self._method_configs['ListPackages'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.ListPackagesRequest(
            name=name,
            view=view,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['list_packages'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def delete_package(self,
                       name,
                       retry=google.api_core.gapic_v1.method.DEFAULT,
                       timeout=google.api_core.gapic_v1.method.DEFAULT,
                       metadata=None):
        """
        Deletes a ``package``. Returns ``NOT_FOUND`` if the matching path does
        not exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.package_path_path('[DEPLOYMENT]', '[PACKAGE_PATH]')
            >>>
            >>> response = client.delete_package(name)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/packages/package\_a"
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.DeletePackageResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'delete_package' not in self._inner_api_calls:
            self._inner_api_calls[
                'delete_package'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.delete_package,
                    default_retry=self._method_configs['DeletePackage'].retry,
                    default_timeout=self._method_configs['DeletePackage'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.DeletePackageRequest(
            name=name, )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['delete_package'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def update_package(self,
                       name,
                       package_obj,
                       version_tag,
                       retry=google.api_core.gapic_v1.method.DEFAULT,
                       timeout=google.api_core.gapic_v1.method.DEFAULT,
                       metadata=None):
        """
        Updates a ``package``. Returns ``INVALID_ARGUMENT`` if any field other
        than title is modified.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.package_path_path('[DEPLOYMENT]', '[PACKAGE_PATH]')
            >>>
            >>> # TODO: Initialize `package_obj`:
            >>> package_obj = {}
            >>>
            >>> # TODO: Initialize `version_tag`:
            >>> version_tag = {}
            >>>
            >>> response = client.update_package(name, package_obj, version_tag)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/packages/package\_a"
            package_obj (Union[dict, ~google.android.overtheair_v1.types.Package]): Required. The ``Package`` to update.

                If a dict is provided, it must be of the same form as the protobuf
                message :class:`~google.android.overtheair_v1.types.Package`
            version_tag (Union[dict, ~google.android.overtheair_v1.types.VersionTag]): Required. The package to be updated should match this version.
                If another client updates the existing config concurrently, the
                update operation will fail.

                If a dict is provided, it must be of the same form as the protobuf
                message :class:`~google.android.overtheair_v1.types.VersionTag`
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.Package` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'update_package' not in self._inner_api_calls:
            self._inner_api_calls[
                'update_package'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.update_package,
                    default_retry=self._method_configs['UpdatePackage'].retry,
                    default_timeout=self._method_configs['UpdatePackage'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.UpdatePackageRequest(
            name=name,
            package_obj=package_obj,
            version_tag=version_tag,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['update_package'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def create_config(self,
                      name,
                      config,
                      retry=google.api_core.gapic_v1.method.DEFAULT,
                      timeout=google.api_core.gapic_v1.method.DEFAULT,
                      metadata=None):
        """
        Creates a config, and returns the new ``Config``.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.deployment_path('[DEPLOYMENT]')
            >>>
            >>> # TODO: Initialize `config`:
            >>> config = {}
            >>>
            >>> response = client.create_config(name, config)

        Args:
            name (str): Relative resource name. For example: "deployments/deployment\_x"
            config (Union[dict, ~google.android.overtheair_v1.types.Config]): Required. The ``Config`` to create.

                If a dict is provided, it must be of the same form as the protobuf
                message :class:`~google.android.overtheair_v1.types.Config`
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.Config` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'create_config' not in self._inner_api_calls:
            self._inner_api_calls[
                'create_config'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.create_config,
                    default_retry=self._method_configs['CreateConfig'].retry,
                    default_timeout=self._method_configs['CreateConfig'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.CreateConfigRequest(
            name=name,
            config=config,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['create_config'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def get_config(self,
                   name,
                   view=None,
                   retry=google.api_core.gapic_v1.method.DEFAULT,
                   timeout=google.api_core.gapic_v1.method.DEFAULT,
                   metadata=None):
        """
        Gets a ``config`` view. Returns ``NOT_FOUND`` if the matching path does
        not exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.config_path('[DEPLOYMENT]', '[CONFIG]')
            >>>
            >>> response = client.get_config(name)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/configs/config\_a"
            view (~google.android.overtheair_v1.types.View): Optional. Specify whether basic or detailed information of the config
                should be returned.
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.Config` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'get_config' not in self._inner_api_calls:
            self._inner_api_calls[
                'get_config'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.get_config,
                    default_retry=self._method_configs['GetConfig'].retry,
                    default_timeout=self._method_configs['GetConfig'].timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.GetConfigRequest(
            name=name,
            view=view,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['get_config'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def list_configs(self,
                     name,
                     view=None,
                     retry=google.api_core.gapic_v1.method.DEFAULT,
                     timeout=google.api_core.gapic_v1.method.DEFAULT,
                     metadata=None):
        """
        Lists ``configs`` of a deployment. Returns ``NOT_FOUND`` if the matching
        path does not exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.deployment_path('[DEPLOYMENT]')
            >>>
            >>> response = client.list_configs(name)

        Args:
            name (str): Relative resource name. For example: "deployments/deployment\_x"
            view (~google.android.overtheair_v1.types.View): Optional. Specify whether basic or detailed information of each config
                should be returned.
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.ListConfigsResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'list_configs' not in self._inner_api_calls:
            self._inner_api_calls[
                'list_configs'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.list_configs,
                    default_retry=self._method_configs['ListConfigs'].retry,
                    default_timeout=self._method_configs['ListConfigs'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.ListConfigsRequest(
            name=name,
            view=view,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['list_configs'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def delete_config(self,
                      name,
                      retry=google.api_core.gapic_v1.method.DEFAULT,
                      timeout=google.api_core.gapic_v1.method.DEFAULT,
                      metadata=None):
        """
        Deletes a ``config``. Returns ``NOT_FOUND`` if the matching path does
        not exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.config_path('[DEPLOYMENT]', '[CONFIG]')
            >>>
            >>> response = client.delete_config(name)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/configs/config\_a"
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.DeleteConfigResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'delete_config' not in self._inner_api_calls:
            self._inner_api_calls[
                'delete_config'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.delete_config,
                    default_retry=self._method_configs['DeleteConfig'].retry,
                    default_timeout=self._method_configs['DeleteConfig'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.DeleteConfigRequest(
            name=name, )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['delete_config'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def update_config(self,
                      name,
                      config,
                      version_tag,
                      retry=google.api_core.gapic_v1.method.DEFAULT,
                      timeout=google.api_core.gapic_v1.method.DEFAULT,
                      metadata=None):
        """
        Updates a ``config``. Returns ``INVALID_ARGUMENT`` if the name of the
        config is non-empty and does not equal the existing name.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.config_path('[DEPLOYMENT]', '[CONFIG]')
            >>>
            >>> # TODO: Initialize `config`:
            >>> config = {}
            >>>
            >>> # TODO: Initialize `version_tag`:
            >>> version_tag = {}
            >>>
            >>> response = client.update_config(name, config, version_tag)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/configs/config\_a"
            config (Union[dict, ~google.android.overtheair_v1.types.Config]): The ``Config`` to update.

                If a dict is provided, it must be of the same form as the protobuf
                message :class:`~google.android.overtheair_v1.types.Config`
            version_tag (Union[dict, ~google.android.overtheair_v1.types.VersionTag]): Required. The config to be updated should match this version.
                If another client updates the existing config concurrently, the
                update operation will fail.

                If a dict is provided, it must be of the same form as the protobuf
                message :class:`~google.android.overtheair_v1.types.VersionTag`
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.Config` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'update_config' not in self._inner_api_calls:
            self._inner_api_calls[
                'update_config'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.update_config,
                    default_retry=self._method_configs['UpdateConfig'].retry,
                    default_timeout=self._method_configs['UpdateConfig'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.UpdateConfigRequest(
            name=name,
            config=config,
            version_tag=version_tag,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['update_config'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def create_group(self,
                     name,
                     group,
                     retry=google.api_core.gapic_v1.method.DEFAULT,
                     timeout=google.api_core.gapic_v1.method.DEFAULT,
                     metadata=None):
        """
        Creates a group, and returns the new ``Group``.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.deployment_path('[DEPLOYMENT]')
            >>>
            >>> # TODO: Initialize `group`:
            >>> group = {}
            >>>
            >>> response = client.create_group(name, group)

        Args:
            name (str): Relative resource name. For example: "deployments/deployment\_x"
            group (Union[dict, ~google.android.overtheair_v1.types.Group]): Required. The ``Group`` to create.

                If a dict is provided, it must be of the same form as the protobuf
                message :class:`~google.android.overtheair_v1.types.Group`
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.Group` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'create_group' not in self._inner_api_calls:
            self._inner_api_calls[
                'create_group'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.create_group,
                    default_retry=self._method_configs['CreateGroup'].retry,
                    default_timeout=self._method_configs['CreateGroup'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.CreateGroupRequest(
            name=name,
            group=group,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['create_group'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def get_group(self,
                  name,
                  view=None,
                  retry=google.api_core.gapic_v1.method.DEFAULT,
                  timeout=google.api_core.gapic_v1.method.DEFAULT,
                  metadata=None):
        """
        Gets a ``group``. Returns ``NOT_FOUND`` if the group does not exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.group_path('[DEPLOYMENT]', '[GROUP]')
            >>>
            >>> response = client.get_group(name)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/groups/group\_g"
            view (~google.android.overtheair_v1.types.View): Optional. Specify whether basic or detailed information of the group
                should be returned.
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.Group` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'get_group' not in self._inner_api_calls:
            self._inner_api_calls[
                'get_group'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.get_group,
                    default_retry=self._method_configs['GetGroup'].retry,
                    default_timeout=self._method_configs['GetGroup'].timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.GetGroupRequest(
            name=name,
            view=view,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['get_group'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def list_groups(self,
                    name,
                    view=None,
                    retry=google.api_core.gapic_v1.method.DEFAULT,
                    timeout=google.api_core.gapic_v1.method.DEFAULT,
                    metadata=None):
        """
        Lists ``groups`` of a deployment. Returns ``NOT_FOUND`` if the matching
        path does not exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.deployment_path('[DEPLOYMENT]')
            >>>
            >>> response = client.list_groups(name)

        Args:
            name (str): Relative resource name. For example: "deployments/deployment\_x"
            view (~google.android.overtheair_v1.types.View): Optional. Specify whether basic or detailed information of each group
                should be returned.
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.ListGroupsResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'list_groups' not in self._inner_api_calls:
            self._inner_api_calls[
                'list_groups'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.list_groups,
                    default_retry=self._method_configs['ListGroups'].retry,
                    default_timeout=self._method_configs['ListGroups'].timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.ListGroupsRequest(
            name=name,
            view=view,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['list_groups'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def delete_group(self,
                     name,
                     retry=google.api_core.gapic_v1.method.DEFAULT,
                     timeout=google.api_core.gapic_v1.method.DEFAULT,
                     metadata=None):
        """
        Deletes a ``group``. Returns ``NOT_FOUND`` if the group does not exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.group_path('[DEPLOYMENT]', '[GROUP]')
            >>>
            >>> response = client.delete_group(name)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/groups/group\_g"
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.DeleteGroupResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'delete_group' not in self._inner_api_calls:
            self._inner_api_calls[
                'delete_group'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.delete_group,
                    default_retry=self._method_configs['DeleteGroup'].retry,
                    default_timeout=self._method_configs['DeleteGroup'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.DeleteGroupRequest(
            name=name, )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['delete_group'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def update_group(self,
                     name,
                     group,
                     version_tag,
                     retry=google.api_core.gapic_v1.method.DEFAULT,
                     timeout=google.api_core.gapic_v1.method.DEFAULT,
                     metadata=None):
        """
        Updates a ``group``. Returns INVALID\_ARGUMENT if the name of the group
        is non-empty and does not equal the existing name.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.group_path('[DEPLOYMENT]', '[GROUP]')
            >>>
            >>> # TODO: Initialize `group`:
            >>> group = {}
            >>>
            >>> # TODO: Initialize `version_tag`:
            >>> version_tag = {}
            >>>
            >>> response = client.update_group(name, group, version_tag)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/groups/group\_g"
            group (Union[dict, ~google.android.overtheair_v1.types.Group]): Required. The ``Group`` to update.

                If a dict is provided, it must be of the same form as the protobuf
                message :class:`~google.android.overtheair_v1.types.Group`
            version_tag (Union[dict, ~google.android.overtheair_v1.types.VersionTag]): Required. The group to be updated should match this version.
                If another client updates the existing config concurrently, the
                update operation will fail.

                If a dict is provided, it must be of the same form as the protobuf
                message :class:`~google.android.overtheair_v1.types.VersionTag`
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.Group` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'update_group' not in self._inner_api_calls:
            self._inner_api_calls[
                'update_group'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.update_group,
                    default_retry=self._method_configs['UpdateGroup'].retry,
                    default_timeout=self._method_configs['UpdateGroup'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.UpdateGroupRequest(
            name=name,
            group=group,
            version_tag=version_tag,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['update_group'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def disable_groups(self,
                       name,
                       group_names,
                       retry=google.api_core.gapic_v1.method.DEFAULT,
                       timeout=google.api_core.gapic_v1.method.DEFAULT,
                       metadata=None):
        """
        Disables the defined ``groups``. Returns ``NOT_FOUND`` if the matching
        path does not exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.deployment_path('[DEPLOYMENT]')
            >>>
            >>> # TODO: Initialize `group_names`:
            >>> group_names = []
            >>>
            >>> response = client.disable_groups(name, group_names)

        Args:
            name (str): Relative resource name. For example: "deployments/deployment\_x/groups"
            group_names (list[str]): Required. The groups that should be disabled.
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.DisableGroupsResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'disable_groups' not in self._inner_api_calls:
            self._inner_api_calls[
                'disable_groups'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.disable_groups,
                    default_retry=self._method_configs['DisableGroups'].retry,
                    default_timeout=self._method_configs['DisableGroups'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.DisableGroupsRequest(
            name=name,
            group_names=group_names,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['disable_groups'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def enable_groups(self,
                      name,
                      group_names,
                      retry=google.api_core.gapic_v1.method.DEFAULT,
                      timeout=google.api_core.gapic_v1.method.DEFAULT,
                      metadata=None):
        """
        Enables the defined ``groups``. Returns ``NOT_FOUND`` if the matching
        path does not exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.deployment_path('[DEPLOYMENT]')
            >>>
            >>> # TODO: Initialize `group_names`:
            >>> group_names = []
            >>>
            >>> response = client.enable_groups(name, group_names)

        Args:
            name (str): Relative resource name. For example: "deployments/deployment\_x/groups"
            group_names (list[str]): Required. The groups should be enabled.
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.EnableGroupsResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'enable_groups' not in self._inner_api_calls:
            self._inner_api_calls[
                'enable_groups'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.enable_groups,
                    default_retry=self._method_configs['EnableGroups'].retry,
                    default_timeout=self._method_configs['EnableGroups'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.EnableGroupsRequest(
            name=name,
            group_names=group_names,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['enable_groups'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def create_group_member(self,
                            name,
                            group_member,
                            retry=google.api_core.gapic_v1.method.DEFAULT,
                            timeout=google.api_core.gapic_v1.method.DEFAULT,
                            metadata=None):
        """
        Creates a group member and returns the new ``GroupMember``. If the
        device is already under other groups, it will be removed from those
        groups and added to the group that's specified in the request.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.group_path('[DEPLOYMENT]', '[GROUP]')
            >>>
            >>> # TODO: Initialize `group_member`:
            >>> group_member = {}
            >>>
            >>> response = client.create_group_member(name, group_member)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/groups/group\_g/members"
            group_member (Union[dict, ~google.android.overtheair_v1.types.GroupMember]): Required. The ``group member`` to create.

                If a dict is provided, it must be of the same form as the protobuf
                message :class:`~google.android.overtheair_v1.types.GroupMember`
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.GroupMember` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'create_group_member' not in self._inner_api_calls:
            self._inner_api_calls[
                'create_group_member'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.create_group_member,
                    default_retry=self._method_configs['CreateGroupMember'].
                    retry,
                    default_timeout=self._method_configs['CreateGroupMember'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.CreateGroupMemberRequest(
            name=name,
            group_member=group_member,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['create_group_member'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def create_group_members(self,
                             name,
                             group_members,
                             retry=google.api_core.gapic_v1.method.DEFAULT,
                             timeout=google.api_core.gapic_v1.method.DEFAULT,
                             metadata=None):
        """
        Creates group members in a specific group.  If any device is in other
        groups, it will be removed from those groups and added to the group that's
        specified in the request.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.group_path('[DEPLOYMENT]', '[GROUP]')
            >>>
            >>> # TODO: Initialize `group_members`:
            >>> group_members = []
            >>>
            >>> response = client.create_group_members(name, group_members)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/groups/group\_g/members"
            group_members (list[Union[dict, ~google.android.overtheair_v1.types.GroupMember]]): Required. The list of ``group member``\ s to create. This list is
                currently limited to no more than 1,000 members. If you need to create
                more than that, please send multiple requests.

                If a dict is provided, it must be of the same form as the protobuf
                message :class:`~google.android.overtheair_v1.types.GroupMember`
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.CreateGroupMembersResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'create_group_members' not in self._inner_api_calls:
            self._inner_api_calls[
                'create_group_members'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.create_group_members,
                    default_retry=self._method_configs['CreateGroupMembers'].
                    retry,
                    default_timeout=self._method_configs['CreateGroupMembers'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.CreateGroupMembersRequest(
            name=name,
            group_members=group_members,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['create_group_members'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def get_group_member(self,
                         name,
                         view=None,
                         retry=google.api_core.gapic_v1.method.DEFAULT,
                         timeout=google.api_core.gapic_v1.method.DEFAULT,
                         metadata=None):
        """
        Gets a ``group member``. Returns ``NOT_FOUND`` if the matching path does
        not exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.member_path('[DEPLOYMENT]', '[GROUP]', '[MEMBER]')
            >>>
            >>> response = client.get_group_member(name)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/groups/group\_g/members/member\_m"
            view (~google.android.overtheair_v1.types.View): Optional. Specify whether basic or detailed information of the group member
                should be returned.
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.GroupMember` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'get_group_member' not in self._inner_api_calls:
            self._inner_api_calls[
                'get_group_member'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.get_group_member,
                    default_retry=self._method_configs['GetGroupMember'].retry,
                    default_timeout=self._method_configs['GetGroupMember'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.GetGroupMemberRequest(
            name=name,
            view=view,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['get_group_member'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def list_group_members(self,
                           name,
                           retry=google.api_core.gapic_v1.method.DEFAULT,
                           timeout=google.api_core.gapic_v1.method.DEFAULT,
                           metadata=None):
        """
        Lists ``group members``. Returns ``NOT_FOUND`` if the matching path does
        not exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.group_path('[DEPLOYMENT]', '[GROUP]')
            >>>
            >>> response = client.list_group_members(name)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/groups/group\_g/members"
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.ListGroupMembersResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'list_group_members' not in self._inner_api_calls:
            self._inner_api_calls[
                'list_group_members'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.list_group_members,
                    default_retry=self._method_configs['ListGroupMembers'].
                    retry,
                    default_timeout=self._method_configs['ListGroupMembers'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.ListGroupMembersRequest(
            name=name, )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['list_group_members'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def delete_group_member(self,
                            name,
                            retry=google.api_core.gapic_v1.method.DEFAULT,
                            timeout=google.api_core.gapic_v1.method.DEFAULT,
                            metadata=None):
        """
        Deletes a ``group member``. Returns ``NOT_FOUND`` if the matching path
        does not exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.member_path('[DEPLOYMENT]', '[GROUP]', '[MEMBER]')
            >>>
            >>> response = client.delete_group_member(name)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/groups/group\_g/members/member\_m"
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.DeleteGroupMemberResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'delete_group_member' not in self._inner_api_calls:
            self._inner_api_calls[
                'delete_group_member'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.delete_group_member,
                    default_retry=self._method_configs['DeleteGroupMember'].
                    retry,
                    default_timeout=self._method_configs['DeleteGroupMember'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.DeleteGroupMemberRequest(
            name=name, )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['delete_group_member'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def delete_group_members(self,
                             name,
                             group_members,
                             retry=google.api_core.gapic_v1.method.DEFAULT,
                             timeout=google.api_core.gapic_v1.method.DEFAULT,
                             metadata=None):
        """
        Deletes group members in a group.  If any member does not it exist, it
        will be ignored.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.group_path('[DEPLOYMENT]', '[GROUP]')
            >>>
            >>> # TODO: Initialize `group_members`:
            >>> group_members = []
            >>>
            >>> response = client.delete_group_members(name, group_members)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/groups/group\_g/members"
            group_members (list[Union[dict, ~google.android.overtheair_v1.types.GroupMember]]): Required. The list of ``group member`` to delete. This list is currently
                limited to no more than 1,000 members. If you need to delete more than
                that, please send multiple requests.

                If a dict is provided, it must be of the same form as the protobuf
                message :class:`~google.android.overtheair_v1.types.GroupMember`
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.DeleteGroupMembersResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'delete_group_members' not in self._inner_api_calls:
            self._inner_api_calls[
                'delete_group_members'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.delete_group_members,
                    default_retry=self._method_configs['DeleteGroupMembers'].
                    retry,
                    default_timeout=self._method_configs['DeleteGroupMembers'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.DeleteGroupMembersRequest(
            name=name,
            group_members=group_members,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['delete_group_members'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def get_group_chart(self,
                        name,
                        options_=None,
                        retry=google.api_core.gapic_v1.method.DEFAULT,
                        timeout=google.api_core.gapic_v1.method.DEFAULT,
                        metadata=None):
        """
        Gets a ``group chart``. Returns ``NOT_FOUND`` if the matching path does
        not exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.chart_path('[DEPLOYMENT]', '[GROUP]', '[CHART]')
            >>>
            >>> response = client.get_group_chart(name)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/groups/group\_g/charts/chart\_c"
            options_ (Union[dict, ~google.android.overtheair_v1.types.ChartOptions]): Optional. Chart Options.

                If a dict is provided, it must be of the same form as the protobuf
                message :class:`~google.android.overtheair_v1.types.ChartOptions`
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.Chart` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'get_group_chart' not in self._inner_api_calls:
            self._inner_api_calls[
                'get_group_chart'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.get_group_chart,
                    default_retry=self._method_configs['GetGroupChart'].retry,
                    default_timeout=self._method_configs['GetGroupChart'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.GetGroupChartRequest(
            name=name,
            options=options_,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['get_group_chart'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def list_group_charts(self,
                          name,
                          options_=None,
                          view=None,
                          retry=google.api_core.gapic_v1.method.DEFAULT,
                          timeout=google.api_core.gapic_v1.method.DEFAULT,
                          metadata=None):
        """
        Lists ``group charts``. Returns ``NOT_FOUND`` if the matching path does
        not exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.group_path('[DEPLOYMENT]', '[GROUP]')
            >>>
            >>> response = client.list_group_charts(name)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/groups/group\_g"
            options_ (Union[dict, ~google.android.overtheair_v1.types.ChartOptions]): Optional. Chart Options.

                If a dict is provided, it must be of the same form as the protobuf
                message :class:`~google.android.overtheair_v1.types.ChartOptions`
            view (~google.android.overtheair_v1.types.View): Optional. Specify whether basic or detailed information of the group chart
                should be returned.  Basic data only includes chart names.
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.ListGroupChartsResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'list_group_charts' not in self._inner_api_calls:
            self._inner_api_calls[
                'list_group_charts'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.list_group_charts,
                    default_retry=self._method_configs['ListGroupCharts'].
                    retry,
                    default_timeout=self._method_configs['ListGroupCharts'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.ListGroupChartsRequest(
            name=name,
            options=options_,
            view=view,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['list_group_charts'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def list_group_configs(self,
                           name,
                           retry=google.api_core.gapic_v1.method.DEFAULT,
                           timeout=google.api_core.gapic_v1.method.DEFAULT,
                           metadata=None):
        """
        Lists configs of the group. Returns ``NOT_FOUND`` if the matching path
        does not exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.group_path('[DEPLOYMENT]', '[GROUP]')
            >>>
            >>> response = client.list_group_configs(name)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/groups/group\_g"
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.ListGroupConfigsResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'list_group_configs' not in self._inner_api_calls:
            self._inner_api_calls[
                'list_group_configs'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.list_group_configs,
                    default_retry=self._method_configs['ListGroupConfigs'].
                    retry,
                    default_timeout=self._method_configs['ListGroupConfigs'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.ListGroupConfigsRequest(
            name=name, )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['list_group_configs'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def update_group_configs(self,
                             name,
                             config_names,
                             version_tag,
                             retry=google.api_core.gapic_v1.method.DEFAULT,
                             timeout=google.api_core.gapic_v1.method.DEFAULT,
                             metadata=None):
        """
        Update the configs under a group. Returns ``NOT_FOUND`` if the matching
        group or config does not exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.group_path('[DEPLOYMENT]', '[GROUP]')
            >>>
            >>> # TODO: Initialize `config_names`:
            >>> config_names = []
            >>>
            >>> # TODO: Initialize `version_tag`:
            >>> version_tag = {}
            >>>
            >>> response = client.update_group_configs(name, config_names, version_tag)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/groups/group\_g"
            config_names (list[str]): Required. The names of the configs under the group.
                Important: the order of configs is important. If a device matches
                multiple sets of packages across configs, the last matching config wins
                and will be used.
            version_tag (Union[dict, ~google.android.overtheair_v1.types.VersionTag]): Required. The group to be updated should match this version.
                If another client updates the existing group concurrently, the
                update operation will fail.

                If a dict is provided, it must be of the same form as the protobuf
                message :class:`~google.android.overtheair_v1.types.VersionTag`
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.UpdateGroupConfigsResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'update_group_configs' not in self._inner_api_calls:
            self._inner_api_calls[
                'update_group_configs'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.update_group_configs,
                    default_retry=self._method_configs['UpdateGroupConfigs'].
                    retry,
                    default_timeout=self._method_configs['UpdateGroupConfigs'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.UpdateGroupConfigsRequest(
            name=name,
            config_names=config_names,
            version_tag=version_tag,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['update_group_configs'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def disable_all_enabled_groups_of_config(
            self,
            name,
            retry=google.api_core.gapic_v1.method.DEFAULT,
            timeout=google.api_core.gapic_v1.method.DEFAULT,
            metadata=None):
        """
        Disables all enabled groups that have the specified config. Returns
        ``NOT_FOUND`` if the matching path does not exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.config_path('[DEPLOYMENT]', '[CONFIG]')
            >>>
            >>> response = client.disable_all_enabled_groups_of_config(name)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/configs/config\_c"
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.DisableAllEnabledGroupsOfConfigResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'disable_all_enabled_groups_of_config' not in self._inner_api_calls:
            self._inner_api_calls[
                'disable_all_enabled_groups_of_config'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.disable_all_enabled_groups_of_config,
                    default_retry=self.
                    _method_configs['DisableAllEnabledGroupsOfConfig'].retry,
                    default_timeout=self.
                    _method_configs['DisableAllEnabledGroupsOfConfig'].timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.DisableAllEnabledGroupsOfConfigRequest(
            name=name, )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['disable_all_enabled_groups_of_config'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def list_config_packages(self,
                             name,
                             retry=google.api_core.gapic_v1.method.DEFAULT,
                             timeout=google.api_core.gapic_v1.method.DEFAULT,
                             metadata=None):
        """
        Lists packages of the config. Returns ``NOT_FOUND`` if the matching path
        does not exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.config_path('[DEPLOYMENT]', '[CONFIG]')
            >>>
            >>> response = client.list_config_packages(name)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/configs/config\_c"
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.ListConfigPackagesResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'list_config_packages' not in self._inner_api_calls:
            self._inner_api_calls[
                'list_config_packages'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.list_config_packages,
                    default_retry=self._method_configs['ListConfigPackages'].
                    retry,
                    default_timeout=self._method_configs['ListConfigPackages'].
                    timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.ListConfigPackagesRequest(
            name=name, )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['list_config_packages'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def update_config_packages(self,
                               name,
                               package_configs,
                               retry=google.api_core.gapic_v1.method.DEFAULT,
                               timeout=google.api_core.gapic_v1.method.DEFAULT,
                               metadata=None):
        """
        Update the packages under the config. Returns ``NOT_FOUND`` if the
        matching path does not exist.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.config_path('[DEPLOYMENT]', '[CONFIG]')
            >>>
            >>> # TODO: Initialize `package_configs`:
            >>> package_configs = []
            >>>
            >>> response = client.update_config_packages(name, package_configs)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/configs/config\_c"
            package_configs (list[Union[dict, ~google.android.overtheair_v1.types.PackageConfig]]): Required. The names of the packages under the config.

                If a dict is provided, it must be of the same form as the protobuf
                message :class:`~google.android.overtheair_v1.types.PackageConfig`
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.UpdateConfigPackagesResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'update_config_packages' not in self._inner_api_calls:
            self._inner_api_calls[
                'update_config_packages'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.update_config_packages,
                    default_retry=self._method_configs['UpdateConfigPackages'].
                    retry,
                    default_timeout=self.
                    _method_configs['UpdateConfigPackages'].timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.UpdateConfigPackagesRequest(
            name=name,
            package_configs=package_configs,
        )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['update_config_packages'](
            request, retry=retry, timeout=timeout, metadata=metadata)

    def list_configs_using_package(
            self,
            name,
            retry=google.api_core.gapic_v1.method.DEFAULT,
            timeout=google.api_core.gapic_v1.method.DEFAULT,
            metadata=None):
        """
        Lists all the configs that use the specified package.

        Example:
            >>> from google.android import overtheair_v1
            >>>
            >>> client = overtheair_v1.OverTheAirServiceClient()
            >>>
            >>> name = client.package_path_path('[DEPLOYMENT]', '[PACKAGE_PATH]')
            >>>
            >>> response = client.list_configs_using_package(name)

        Args:
            name (str): Relative resource name. For example:
                "deployments/deployment\_x/packages/package\_a"
            retry (Optional[google.api_core.retry.Retry]):  A retry object used
                to retry requests. If ``None`` is specified, requests will not
                be retried.
            timeout (Optional[float]): The amount of time, in seconds, to wait
                for the request to complete. Note that if ``retry`` is
                specified, the timeout applies to each individual attempt.
            metadata (Optional[Sequence[Tuple[str, str]]]): Additional metadata
                that is provided to the method.

        Returns:
            A :class:`~google.android.overtheair_v1.types.ListConfigsUsingPackageResponse` instance.

        Raises:
            google.api_core.exceptions.GoogleAPICallError: If the request
                    failed for any reason.
            google.api_core.exceptions.RetryError: If the request failed due
                    to a retryable error and retry attempts failed.
            ValueError: If the parameters are invalid.
        """
        # Wrap the transport method to add retry and timeout logic.
        if 'list_configs_using_package' not in self._inner_api_calls:
            self._inner_api_calls[
                'list_configs_using_package'] = google.api_core.gapic_v1.method.wrap_method(
                    self.transport.list_configs_using_package,
                    default_retry=self.
                    _method_configs['ListConfigsUsingPackage'].retry,
                    default_timeout=self.
                    _method_configs['ListConfigsUsingPackage'].timeout,
                    client_info=self._client_info,
                )

        request = overtheair_service_messages_pb2.ListConfigsUsingPackageRequest(
            name=name, )
        if metadata is None:
            metadata = []
        metadata = list(metadata)
        try:
            routing_header = [('name', name)]
        except AttributeError:
            pass
        else:
            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
                routing_header)
            metadata.append(routing_metadata)

        return self._inner_api_calls['list_configs_using_package'](
            request, retry=retry, timeout=timeout, metadata=metadata)
